/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: SlsPreviewBitmapFactory.cxx,v $
 *
 *  $Revision: 1.6 $
 *
 *  last change: $Author: rt $ $Date: 2005/09/09 06:10:59 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#include "SlsPreviewBitmapFactory.hxx"
#include "Window.hxx"
#include "view/SlideSorterView.hxx"
#include "view/SlsPageObjectViewObjectContact.hxx"
#include <svx/svdpagv.hxx>
#include <svx/sdr/contact/displayinfo.hxx>
#include <svx/sdr/contact/viewcontact.hxx>

using namespace ::sdr::contact;
using namespace ::sd::slidesorter::view;

namespace sd { namespace slidesorter { namespace cache {

PreviewBitmapFactory::PreviewBitmapFactory (SlideSorterView& rView)
    : mrView (rView),
      mpInfoRectangle (NULL)
{
}




BitmapEx PreviewBitmapFactory::CreateBitmap (
    PageObjectViewObjectContact& rContact)
{
    // Set up an environment for rendering the preview bitmap.
    const SdrPage* pPage = static_cast<PageObjectViewObjectContact&>(
        rContact).GetPage();
    SdrPageView* pPageView = mrView.GetPageViewPvNum(0);

    BitmapEx aBitmap;
    if (pPageView != NULL)
    {
        // Create a display info for painting the preview bitmap.
        
        DisplayInfo aDisplayInfo (pPageView);
        aDisplayInfo.SetProcessLayers (pPageView->GetVisibleLayers());
        aDisplayInfo.SetExtendedOutputDevice(
            mrView.GetExtendedOutputDevice());

        Rectangle aDirtyRectangle (
            rContact.GetViewContact().GetPaintRectangle() 
            - pPageView->GetOffset());
        ProvideInfoRectangle (aDirtyRectangle, pPageView);
        aDisplayInfo.SetPaintInfoRec (mpInfoRectangle.get());
        aDisplayInfo.SetOutputDevice(mrView.GetWindow());

        Rectangle aPaintBox (
            rContact.GetViewContact().GetPaintRectangle());
        aDisplayInfo.SetRedrawArea(Region(aPaintBox));

        aDisplayInfo.SetPreRenderingAllowed(sal_False);
        aDisplayInfo.SetPagePainting(sal_True);
		
        try
        {
            aBitmap = rContact.CreatePreview (aDisplayInfo);
        }
        catch (...)
        {
            DBG_ASSERT (false, "SlsPreviewBitmapFactory::CreateBitmap() "
                "caught exception when asking its view object contact "
                "for a bitmap.");
        }
    }

    return aBitmap;
}




void PreviewBitmapFactory::ProvideInfoRectangle (
    const Rectangle& rDirtyRect, 
    SdrPageView* pPageView)
{
	SdrPaintInfoRec* pInfoRec = new SdrPaintInfoRec();
    sal_Bool bPrinter = sal_False;//( OUTDEV_PRINTER == mrOutputDevice.GetOutDevType());
	Rectangle aCheckRect(rDirtyRect);
    OutputDevice& rDevice (*mrView.GetWindow());
	Size a1PixSiz(rDevice.PixelToLogic(Size(1, 1)));
	const SdrView& rSdrView = pPageView->GetView();
	sal_uInt16 nPaintMode = 0;
    const SdrLayerID* pId = NULL;

	aCheckRect.Left() -= a1PixSiz.Width();
	aCheckRect.Top() -= a1PixSiz.Height();
	aCheckRect.Right() += a1PixSiz.Width();
	aCheckRect.Bottom() += a1PixSiz.Height();

	pInfoRec->pPV = pPageView;
	pInfoRec->bPrinter = bPrinter;
	pInfoRec->aDirtyRect = rDirtyRect + pPageView->GetOffset();
	pInfoRec->aCheckRect = aCheckRect;

	if(bPrinter)
	{
		if(rSdrView.IsLineDraftPrn()) nPaintMode |= SDRPAINTMODE_DRAFTLINE;
		if(rSdrView.IsFillDraftPrn()) nPaintMode |= SDRPAINTMODE_DRAFTFILL;
		if(rSdrView.IsTextDraftPrn()) nPaintMode |= SDRPAINTMODE_DRAFTTEXT;
		if(rSdrView.IsGrafDraftPrn()) nPaintMode |= SDRPAINTMODE_DRAFTGRAF;
	}
	else
	{
		if(rSdrView.IsLineDraft()) nPaintMode |= SDRPAINTMODE_DRAFTLINE;
		if(rSdrView.IsFillDraft()) nPaintMode |= SDRPAINTMODE_DRAFTFILL;
		if(rSdrView.IsTextDraft()) nPaintMode |= SDRPAINTMODE_DRAFTTEXT;
		if(rSdrView.IsGrafDraft()) nPaintMode |= SDRPAINTMODE_DRAFTGRAF;
		if(rSdrView.IsHideGrafDraft()) nPaintMode |= SDRPAINTMODE_HIDEDRAFTGRAF;
	}

	if(pId)
	{
		pInfoRec->aPaintLayer.ClearAll();
		pInfoRec->aPaintLayer.Set(*pId);
	}
	else
	{
		pInfoRec->aPaintLayer = bPrinter 
			? pPageView->GetPrintableLayers() 
			: pPageView->GetVisibleLayers();
	}

	pInfoRec->nPaintMode = nPaintMode;

	if(pPageView->GetObjList() != pPageView->GetPage())
	{
		pInfoRec->pAktList = pPageView->GetObjList();
	}

    mpInfoRectangle.reset (pInfoRec);
}




} } } // end of namespace ::sd::slidesorter::cache
